// Format: '@exp:2025-06-15' or '@exp:+7d' (or '@expires')
const EXPIRATION_DATE_REGEX = /^(@(?:exp|expires):(?:(?:\d{4}-\d{2}-\d{2})|(?:\+\d+[dwmy])))\s*(.*)/

export const formatDateLong = (dateString: string | Date): string => {
	if (!dateString) {
		return ""
	}

	const d = new Date(dateString)
	return d.toLocaleDateString(undefined, {
		weekday: "short",
		year: "2-digit",
		month: "short",
		day: "numeric",
		hour: "numeric",
		minute: "numeric"
	})
}

export const formatDateShort = (input: string | Date): string => {
	console.log(input)

	if (!input) {
		return ""
	}

	const d = input instanceof Date ? input : new Date(input)
	const utcDate = new Date(d.getTime())

	return `${utcDate.toLocaleString("en", { weekday: "short", day: "numeric", month: "short", year: "2-digit", timeZone: "UTC" })} UTC`
}

export const parseExpirationPrefix = (title: string): [string, string] => {
	const match = title.match(EXPIRATION_DATE_REGEX)

	if (match && match[1]) {
		// console.log(`[UTIL] Extracted expiration prefix: '${match[1]}'`)
		// return [clean title, expiration prefix]
		return [match[2], match[1]]
	}

	return [title, ""]
}

export const formatTitleWithHighlight = (title: string): string => {
	const [cleanTitle, expirationPrefix] = parseExpirationPrefix(title)
	if (expirationPrefix === "") {
		return title
	}

	return `<span class="note-title-exp-highlight">${expirationPrefix}</span> ${cleanTitle}`
}

export const clickOutside = (
	node: HTMLElement,
	callbackFunction: () => void
): {
	destroy: () => void
} => {
	const handleClick = (event: MouseEvent): void => {
		if (node && !node.contains(event.target as Node) && !event.defaultPrevented) {
			callbackFunction()
		}
	}

	// event listener with capture phase to ensure it runs before other click handlers
	document.addEventListener("click", handleClick, true)

	return {
		destroy: () => {
			document.removeEventListener("click", handleClick, true)
		}
	}
}
